<?php

namespace Drupal\smart_date\Plugin\Field\FieldFormatter;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Form\FormStateInterface;
// use Drupal\datetime\Plugin\Field\FieldFormatter\TimestampCustomFormatter;
use Drupal\Core\Field\Plugin\Field\FieldFormatter\TimestampFormatter;
use Drupal\smart_date\SmartDateTrait;

/**
 * Plugin implementation of the 'Custom' formatter for 'smartdate' fields.
 *
 * This formatter renders the time range as plain text, with a fully
 * configurable date format using the PHP date syntax and separator.
 *
 * @FieldFormatter(
 *   id = "smartdate_custom",
 *   label = @Translation("Custom"),
 *   field_types = {
 *     "smartdate"
 *   }
 * )
 */
class SmartDateCustomFormatter extends TimestampFormatter {

  use SmartDateTrait;

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
      'separator' => '-',
    ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    // @TODO evaluate using strings instead to allow for more flexible
    // presentation, including a time range on the same day
    $elements = [];
    $separator = $this->getSetting('separator');

    foreach ($items as $delta => $item) {
      if (!empty($item->start_time) && !empty($item->end_time)) {
        /** @var \Drupal\Core\Datetime\DrupalDateTime $start_time */
        $start_time = $item->start_time;
        /** @var \Drupal\Core\Datetime\DrupalDateTime $end_time */
        $end_time = $item->end_time;

        if ($start_time->getTimestamp() !== $end_time->getTimestamp()) {
          $elements[$delta] = [
            'start_time' => $this->buildDate($start_time),
            'separator' => ['#plain_text' => ' ' . $separator . ' '],
            'end_time' => $this->buildDate($end_time),
          ];
        }
        else {
          $elements[$delta] = $this->buildDate($start_time);
        }
      }
    }

    return $elements;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $form = parent::settingsForm($form, $form_state);

    $form['separator'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Date separator'),
      '#description' => $this->t('The string to separate the start and end dates'),
      '#default_value' => $this->getSetting('separator'),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = parent::settingsSummary();

    if ($separator = $this->getSetting('separator')) {
      $summary[] = $this->t('Separator: %separator', ['%separator' => $separator]);
    }

    return $summary;
  }

}
