<?php

namespace Drupal\smart_date\Plugin\Field\FieldType;

use Drupal\Core\Datetime\DrupalDateTime;
use Drupal\Core\Entity\FieldableEntityInterface;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemList;
use Drupal\Core\Form\FormStateInterface;
use Drupal\datetime\Plugin\Field\FieldType\DateTimeFieldItemList;
use Drupal\datetime\Plugin\Field\FieldType\DateTimeItemInterface;

/**
 * Represents a configurable entity smartdate field.
 */
class SmartDateFieldItemList extends DateTimeFieldItemList {

  /**
   * {@inheritdoc}
   */
  public function defaultValuesForm(array &$form, FormStateInterface $form_state) {
    if (empty($this->getFieldDefinition()->getDefaultValueCallback())) {
      if ($this->getFieldDefinition()->getDefaultValueLiteral()) {
        $default_value = $this->getFieldDefinition()->getDefaultValueLiteral()[0];
      }
      else {
        $default_value = array();
      }

      $element = parent::defaultValuesForm($form, $form_state);

      unset($element['default_time_type']);

      $element['default_duration_increments'] = [
        '#type' => 'textfield',
        '#title' => $this->t('Duration increments'),
        '#description' => $this->t('Provide a comma-separated list of valid duration values. Add "custom" to allow an arbitrary length.'),
        '#default_value' => isset($default_value['default_duration_increments']) ? $default_value['default_duration_increments'] : '30, 60, 90, 120, custom',
        '#required' => TRUE,
      ];

      $element['default_duration'] = [
        '#type' => 'textfield',
        '#title' => $this->t('Default duration'),
        '#description' => $this->t('Set which of the duration increments provided above that should be selected by default.'),
        '#default_value' => isset($default_value['default_duration']) ? $default_value['default_duration'] : '60',
        '#required' => TRUE,
      ];

      return $element;
    }
  }

  /**
   * {@inheritdoc}
   */
  public function defaultValuesFormValidate(array $element, array &$form, FormStateInterface $form_state) {
    if ($duration = $form_state->getValue(['default_value_input', 'default_duration'])) {
      $increments = explode(',', $form_state->getValue(['default_value_input', 'default_duration_increments']));
      $increment_sum = 0;
      foreach ($increments as $delta => $increment) {
        $increments[$delta] = trim($increment);
        if (intval($increments[$delta]) == 0 && $increments[$delta] !== 'custom') {
          $form_state->setErrorByName('default_value_input][default_duration_increments', $this->t('Invalid tokens in the provided increments string. Please provide either integers or "custom" for each value.'));
          break;
        }
        $increment_sum += intval($increments[$delta]);
      }
      if (!in_array('custom', $increments)) {
        if ($increment_sum <= 0) {
          $form_state->setErrorByName('default_value_input][default_duration_increments', $this->t('Unable to parse valid durations from the provided increments string.'));
        } else {
          $messenger = \Drupal::messenger();
          $messenger->addMessage($this->t('No string to allow for custom values, so the provided increments will be strictly enforced.'), 'warning');
        }
      }
      if (!in_array($duration, $increments)) {
        $form_state->setErrorByName('default_value_input][default_duration', $this->t('Please specify a default duration that is one of the provided options.'));
      }
    }
    // Use the parent class method to validate relative dates.
    DateTimeFieldItemList::defaultValuesFormValidate($element, $form, $form_state);
    // if ($date_type = $form_state->getValue(['default_value_input', 'default_date_type'])) {
    //   if ($date_type == 'relative') {
    //     $relative_date = $form_state->getValue(['default_value_input', 'default_date']);
    //     if (@strtotime($relative_date) === FALSE) {
    //       $form_state->setErrorByName('default_value_input][default_date', $this->t('Unable to interpret relative date string provided.'));
    //     }
    //   }
    // }
  }

  /**
   * {@inheritdoc}
   */
  public function defaultValuesFormSubmit(array $element, array &$form, FormStateInterface $form_state) {
    if ($form_state->getValue(['default_value_input', 'default_duration']) && $form_state->getValue(['default_value_input', 'default_duration_increments'])) {
      if ($duration = $form_state->getValue(['default_value_input', 'default_duration'])) {
        $form_state->setValueForElement($element['default_duration'], $duration);
      }
      if ($duration_increments = $form_state->getValue(['default_value_input', 'default_duration_increments'])) {
        $form_state->setValueForElement($element['default_duration_increments'], $duration_increments);
      }
      return [$form_state->getValue('default_value_input')];
    }
    // Use the parent class method to store "current date configuration".
    DateTimeFieldItemList::defaultValuesFormSubmit($element, $form, $form_state);
    return [];
  }

  /**
   * {@inheritdoc}
   */
  public static function processDefaultValue($default_value, FieldableEntityInterface $entity, FieldDefinitionInterface $definition) {
    // Explicitly call the base class so that we can get the default value
    // types.
    $default_value = FieldItemList::processDefaultValue($default_value, $entity, $definition);

    // No default set, so nothing to do.
    if (empty($default_value[0]['default_date_type'])) {
      return $default_value;
    }

    // A default date+time value should be in the format and timezone used
    // for date storage.
    $date = new DrupalDateTime($default_value[0]['default_date'], DateTimeItemInterface::STORAGE_TIMEZONE);
    $format = DateTimeItemInterface::DATETIME_STORAGE_FORMAT;

    $value = $date->getTimestamp();
    // Round up to the next minute.
    $second = $date->format("s");
    if($second > 0) {
      $value += 60 - $second;
    }
    // Calculate the end value.
    $duration = (int) $default_value[0]['default_duration'];
    $end_value = $value + ($duration * 60);

    $default_value = [
      [
        'value' => $value,
        'end_value' => $end_value,
        'date' => $date,
      ],
    ];

    return $default_value;
  }

}
